@@ -11,7 +11,6 @@ use clap::Parser;
 use clap::ValueEnum;
 use git_version::git_version;
 use line_collector::LineCollector;
-use log::error;
 use logging::init_logger;
 use refiner::Formatter;
 use std::io::{self, IsTerminal};
@@ -26,12 +25,12 @@ mod ansi;
 mod commit_line;
 mod conflicts_highlighter;
 mod constants;
+mod file_highlighter;
 mod hunk_header;
 mod hunk_highlighter;
 mod line_collector;
 mod lines_highlighter;
 mod logging;
-mod plusminus_header_highlighter;
 mod plusminus_lines_highlighter;
 mod refiner;
 mod rename_highlighter;
@@ -139,15 +138,12 @@ impl ColorOption {
 /// How will unchanged line parts be styled?
 #[derive(ValueEnum, Clone, Default, Debug)]
 pub(crate) enum UnchangedStyle {
-    /// No special styling, just red / green like the rest of the line
+    /// Unchanged text is yellow, old unchanged is faint
     #[default]
-    None,
-
-    /// Old unchanged faint yellow, new unchanged plain yellow
     Yellow,
 
-    /// Try it and report back! Same as yellow for now.
-    Experimental,
+    /// Legacy mode
+    RedGreen,
 }
 
 fn format_error(message: String, line_number: usize, line: &[u8]) -> String {
@@ -206,7 +202,7 @@ fn highlight_diff<W: io::Write + Send + 'static>(
                 if let Err(message) =
                     line_collector.consume_line(&line, stream_started_with_esc.unwrap_or(false))
                 {
-                    error!("{}", format_error(message, line_number, &line));
+                    log::error!("{}", format_error(message, line_number, &line));
                 }
             }
             break;
@@ -228,7 +224,7 @@ fn highlight_diff<W: io::Write + Send + 'static>(
             if let Err(message) =
                 line_collector.consume_line(&line, stream_started_with_esc.unwrap_or(false))
             {
-                error!("{}", format_error(message, line_number, &line));
+                log::error!("{}", format_error(message, line_number, &line));
             }
             line.clear();
             line_number += 1;
@@ -516,10 +512,9 @@ fn main() {
         panic!("Panicking on purpose");
     }
 
-    let formatter = match options.unchanged_style.unwrap_or(UnchangedStyle::None) {
-        UnchangedStyle::None => Formatter::default(),
+    let formatter = match options.unchanged_style.unwrap_or(UnchangedStyle::Yellow) {
+        UnchangedStyle::RedGreen => Formatter::default(),
         UnchangedStyle::Yellow => Formatter::yellow(),
-        UnchangedStyle::Experimental => Formatter::yellow(),
     };
 
     if let (Some(file1), Some(file2)) = (options.fd1, options.fd2) {
@@ -605,24 +600,29 @@ mod tests {
     #[cfg(test)]
     use pretty_assertions::assert_eq;
 
-    fn new(text: &str) -> String {
-        return format!("{GREEN}{text}{NORMAL}");
-    }
-
     #[test]
     fn test_trailing_newline_context() {
-        let mut input = "@@ -1,1 +1,2 @@\n+bepa\n apa\n\\ No newline at end of file\n".as_bytes();
+        let mut input = "--- a/foo.txt\n+++ b/foo.txt\n@@ -1,1 +1,2 @@\n+bepa\n apa\n\\ No newline at end of file\n".as_bytes();
 
-        let expected = format!(
-            "{}{}{}\n{}\n{}\n{}\\ No newline at end of file{}\n",
-            HUNK_HEADER,
-            "@@ -1,1 +1,2 @@",
-            NORMAL,
-            new("+bepa"),
-            " apa",
-            NO_EOF_NEWLINE_COLOR,
-            NORMAL
-        );
+        let expected = [
+            format!(
+                "{}--- {}{}a/{}{}foo.txt{}",
+                BOLD, NORMAL_INTENSITY, FAINT, NORMAL_INTENSITY, BOLD, NORMAL
+            ),
+            format!(
+                "{}+++ {}{}b/{}{}foo.txt{}",
+                BOLD, NORMAL_INTENSITY, FAINT, NORMAL_INTENSITY, BOLD, NORMAL
+            ),
+            format!("{}@@ -1,1 +1,2 @@{}", HUNK_HEADER, NORMAL),
+            format!("{}+bepa{}", GREEN, NORMAL),
+            " apa".to_string(),
+            format!(
+                "{}\\ No newline at end of file{}",
+                NO_EOF_NEWLINE_COLOR, NORMAL
+            ),
+        ]
+        .join("\n")
+            + "\n";
 
         let file = tempfile::NamedTempFile::new().unwrap();
         if let Err(error) = highlight_diff(
